#!/usr/bin/env python3
import asyncio

from modules.write_log import log_writer

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow


async def hackertgt(hostname, session):
    subdomains = []
    error = None
    url = f"https://api.hackertarget.com/hostsearch/?q={hostname}"
    try:
        async with session.get(url) as resp:
            status = resp.status
            if status == 200:
                data = await resp.text()
                data_list = data.split("\n")
                tmp_list = []
                for line in data_list:
                    subdomain = line.split(",")[0]
                    tmp_list.append(subdomain)
                subdomains.extend(tmp_list)
            else:
                error = f"Status : {status}"
                log_writer(f"[htarget_subs] Status = {status}, expected 200")
    except asyncio.TimeoutError:
        error = "Request Timeout"
        log_writer(f"[htarget_subs] Exception = {error}")
    except Exception as exc:
        error = exc
        log_writer(f"[htarget_subs] Exception = {exc}")
    log_writer("[htarget_subs] Completed")

    return "Hackertarget", subdomains, error



#!/usr/bin/env python3

from json import loads
import modules.subdom as parent
from modules.write_log import log_writer

R = '\033[31m'  # red
G = '\033[32m'  # green
C = '\033[36m'  # cyan
W = '\033[0m'   # white
Y = '\033[33m'  # yellow


async def thcrowd(hostname, session):
	print(f'{Y}[!] {C}Requesting {G}ThreatCrowd{W}')
	url = 'https://www.threatcrowd.org/searchApi/v2/domain/report/'
	thc_params = {
		'domain': hostname
	}
	try:
		async with session.get(url, params=thc_params) as resp:
			status = resp.status
			if status == 200:
				output = await resp.text()
				json_out = loads(output)
				if json_out['response_code'] == '0':
					pass
				else:
					subd = json_out['subdomains']
					print(f'{G}[+] {Y}ThreatCrowd {W}found {C}{len(subd)} {W}subdomains!')
					parent.found.extend(subd)
			else:
				print(f'{R}[-] {C}ThreatCrowd Status : {W}{status}')
				log_writer(f'[thcrowd] Status = {status}, expected 200')
	except Exception as exc:
		print(f'{R}[-] {C}ThreatCrowd Exception : {W}{exc}')
		log_writer(f'[thcrowd] Exception = {exc}')
	log_writer('[thcrowd] Completed')



#!/usr/bin/env python3

import os
import sys

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white

from modules.write_log import log_writer

log_writer("Importing config...")
import settings as config

home = config.home
usr_data = config.usr_data
conf_path = config.conf_path
path_to_script = config.path_to_script
src_conf_path = config.src_conf_path
meta_file_path = config.meta_file_path
keys_file_path = config.keys_file_path

log_writer(
    f"PATHS = HOME:{home}, SCRIPT_LOC:{path_to_script},\
    METADATA:{meta_file_path}, KEYS:{config.keys_file_path},\
    CONFIG:{config.conf_file_path}, LOG:{config.log_file_path}"
)

import argparse

VERSION = config.version
log_writer(f"FinalRecon v{VERSION}")

parser = argparse.ArgumentParser(
    description=f"FinalRecon - All in One Web Recon | v{VERSION}"
)
parser.add_argument("--url", help="Target URL")
parser.add_argument("--headers", help="Header Information", action="store_true")
parser.add_argument(
    "--sslinfo", help="SSL Certificate Information", action="store_true"
)
parser.add_argument("--whois", help="Whois Lookup", action="store_true")
parser.add_argument("--crawl", help="Crawl Target", action="store_true")
parser.add_argument("--dns", help="DNS Enumeration", action="store_true")
parser.add_argument("--sub", help="Sub-Domain Enumeration", action="store_true")
parser.add_argument("--dir", help="Directory Search", action="store_true")
parser.add_argument("--wayback", help="Wayback URLs", action="store_true")
parser.add_argument("--ps", help="Fast Port Scan", action="store_true")
parser.add_argument("--full", help="Full Recon", action="store_true")

ext_help = parser.add_argument_group("Extra Options")
ext_help.add_argument("-nb", action="store_false", help="Hide Banner")
ext_help.add_argument(
    "-dt", type=int, help="Number of threads for directory enum [ Default : 30 ]"
)
ext_help.add_argument(
    "-pt", type=int, help="Number of threads for port scan [ Default : 50 ]"
)
ext_help.add_argument("-T", type=float, help="Request Timeout [ Default : 30.0 ]")
ext_help.add_argument(
    "-w", help="Path to Wordlist [ Default : wordlists/dirb_common.txt ]"
)
ext_help.add_argument(
    "-r", action="store_true", help="Allow Redirect [ Default : False ]"
)
ext_help.add_argument(
    "-s", action="store_false", help="Toggle SSL Verification [ Default : True ]"
)
ext_help.add_argument("-sp", type=int, help="Specify SSL Port [ Default : 443 ]")
ext_help.add_argument("-d", help="Custom DNS Servers [ Default : 1.1.1.1 ]")
ext_help.add_argument("-e", help="File Extensions [ Example : txt, xml, php ]")
ext_help.add_argument("-o", help="Export Format [ Default : txt ]")
ext_help.add_argument(
    "-cd", help="Change export directory [ Default : ~/.local/share/finalrecon ]"
)
ext_help.add_argument(
    "-of", help="Change export folder name [ Default :<path>fr_<hostname>_<date> ]"
)
ext_help.add_argument("-k", help="Add API key [ Example : shodan@key ]")
ext_help.set_defaults(
    dt=config.dir_enum_th,
    pt=config.port_scan_th,
    T=config.timeout,
    w=config.dir_enum_wlist,
    r=config.dir_enum_redirect,
    s=config.dir_enum_sslv,
    sp=config.ssl_port,
    d=config.custom_dns,
    e=config.dir_enum_ext,
    o=config.export_fmt,
    cd=config.usr_data,
    of=None,
)

try:
    args = parser.parse_args()
except SystemExit:
    log_writer("[finalrecon] Help menu accessed")
    log_writer(f"{'-' * 30}")
    sys.exit()

target = args.url
headinfo = args.headers
sslinfo = args.sslinfo
whois = args.whois
crawl = args.crawl
dns = args.dns
dirrec = args.dir
wback = args.wayback
pscan = args.ps
full = args.full
threads = args.dt
pscan_threads = args.pt
tout = args.T
wdlist = args.w
redir = args.r
sslv = args.s
sslp = args.sp
dserv = args.d
filext = args.e
subd = args.sub
output = args.o
show_banner = args.nb
add_key = args.k
output_dir = args.cd
folder_name = args.of

import datetime
import ipaddress
import socket
from json import dumps, loads
from urllib import parse

import tldextract

type_ip = False
data = {}


def banner():
    with open(meta_file_path, "r") as metadata:
        json_data = loads(metadata.read())
        twitter_url = json_data["twitter"]
        comms_url = json_data["comms"]

    art = r"""
 ______  __   __   __   ______   __
/\  ___\/\ \ /\ "-.\ \ /\  __ \ /\ \
\ \  __\\ \ \\ \ \-.  \\ \  __ \\ \ \____
 \ \_\   \ \_\\ \_\\"\_\\ \_\ \_\\ \_____\
  \/_/    \/_/ \/_/ \/_/ \/_/\/_/ \/_____/
 ______   ______   ______   ______   __   __
/\  == \ /\  ___\ /\  ___\ /\  __ \ /\ "-.\ \
\ \  __< \ \  __\ \ \ \____\ \ \/\ \\ \ \-.  \
 \ \_\ \_\\ \_____\\ \_____\\ \_____\\ \_\\"\_\
  \/_/ /_/ \/_____/ \/_____/ \/_____/ \/_/ \/_/"""
    print(f"{C}{art}{W}\n")
    print(f"{G}[>]{W} Created By   : thewhiteh4t")
    print(f"{G} |--->{W} Twitter   : {twitter_url}")
    print(f"{G} |--->{W} Community : {comms_url}")
    print(f"{G}[>]{W} Version      : {VERSION}\n")


def save_key(key_string):
    valid_keys = [
        "bevigil",
        "netlas",
        "shodan",
        "virustotal",
        "zoomeye",
        "hunter",
        "alienvault",
        "chaos",
        "leakix",
        "github",
    ]
    key_parts = key_string.split("@", 1)
    key_name = key_parts[0]
    key_str = key_parts[1]
    if key_name not in valid_keys:
        print(f"{R}[-]{W} Invalid key name!")
        log_writer("Invalid key name, exiting")
        sys.exit(1)
    with open(keys_file_path, "r") as keyfile:
        keys_json = loads(keyfile.read())
    keys_json[key_name] = key_str
    with open(keys_file_path, "w") as key_update:
        key_update.write(dumps(keys_json))
    print(f"{G}[+]{W} {key_name} Key Added!")
    sys.exit(1)


try:
    if show_banner:
        banner()

    if add_key:
        save_key(add_key)

    if not target:
        print(f"{R}[-]{W} No Target Specified!")
        sys.exit(1)

    if not target.startswith(("http", "https")):
        print(f"{R}[-]{W} Protocol Missing, Include {C}http://{W} or {C}https://{W} \n")
        log_writer(f"Protocol missing in {target}, exiting")
        sys.exit(1)

    if target.endswith("/"):
        target = target[:-1]

    print(f"{C}[*]{W} Target : {target}")

    split_url = parse.urlsplit(target)
    protocol = split_url.scheme
    hostname = split_url.hostname

    try:
        ipaddress.ip_address(hostname)
        type_ip = True
        ip = hostname
    except ValueError:
        try:
            ip = socket.gethostbyname(hostname)
            print(f"\n{C}[*]{W} IP Address : {str(ip)}")
        except Exception as e:
            print(f"\n{R}[-]{W} Unable to Get IP : {str(e)}")
            sys.exit(1)

    private_ip = ipaddress.ip_address(ip).is_private

    port = split_url.port
    
    if type_ip:
        netloc = f"{hostname}:{port}" if port else hostname
        domain = ""
        domain_suffix = ""
        apex_domain = ""
    elif not private_ip:
        extractor = tldextract.TLDExtract()
        parsed_url = extractor.extract_urllib(split_url)
        domain = parsed_url.domain
        domain_suffix = parsed_url.suffix
        parsed_fqdn = parsed_url.fqdn if parsed_url.fqdn else hostname
        netloc = f"{parsed_fqdn}:{port}" if port else parsed_fqdn
        apex_domain = f"{domain}.{domain_suffix}"
    else:
        netloc = f"{hostname}:{port}" if port else hostname
        domain = ""
        domain_suffix = ""
        apex_domain = ""
        
    start_time = datetime.datetime.now()

    if output != "None":
        fpath = output_dir
        if not folder_name:
            dt_now = str(datetime.datetime.now().strftime("%d-%m-%Y_%H:%M:%S"))
            fname = f"{fpath}fr_{hostname}_{dt_now}.{output}"
            respath = f"{fpath}fr_{hostname}_{dt_now}"
        else:
            fname = f"{fpath}{folder_name}.{output}"
            respath = f"{fpath}{folder_name}"
        if not os.path.exists(respath):
            os.makedirs(respath)
        out_settings = {"format": output, "directory": respath, "file": fname}
        log_writer(f"OUTPUT = FORMAT: {output}, DIR: {respath}, FILENAME: {fname}")

    if full:
        log_writer("Starting full recon...")

        from modules.crawler import crawler
        from modules.dirrec import hammer
        from modules.dns import dnsrec
        from modules.headers import headers
        from modules.portscan import scan
        from modules.sslinfo import cert

        headers(target, out_settings, data)
        cert(hostname, sslp, out_settings, data)
        dnsrec(apex_domain, dserv, out_settings, data)
        if not type_ip and not private_ip:
            from modules.subdom import subdomains
            from modules.wayback import timetravel
            from modules.whois import whois_lookup

            whois_lookup(domain, domain_suffix, path_to_script, out_settings, data)
            subdomains(apex_domain, tout, out_settings, data, conf_path)
            timetravel(target, data, out_settings)
        scan(ip, out_settings, data, pscan_threads)
        crawler(target, protocol, netloc, out_settings, data)
        hammer(target, threads, tout, wdlist, redir, sslv, out_settings, data, filext)

    if headinfo:
        from modules.headers import headers

        log_writer("Starting header enum...")
        headers(target, out_settings, data)

    if sslinfo:
        from modules.sslinfo import cert

        log_writer("Starting SSL enum...")
        cert(hostname, sslp, out_settings, data)

    if whois:
        from modules.whois import whois_lookup

        log_writer("Starting whois enum...")
        whois_lookup(domain, domain_suffix, path_to_script, out_settings, data)

    if crawl:
        from modules.crawler import crawler

        log_writer("Starting crawler...")
        crawler(target, protocol, netloc, out_settings, data)

    if dns:
        from modules.dns import dnsrec

        log_writer("Starting DNS enum...")
        dnsrec(hostname, dserv, out_settings, data)

    if subd and not type_ip and not private_ip:
        from modules.subdom import subdomains

        log_writer("Starting subdomain enum...")
        subdomains(hostname, tout, out_settings, data, conf_path)

    elif subd and type_ip:
        print(f"{R}[-]{W} Sub-Domain Enumeration is Not Supported for IP Addresses\n")
        log_writer("Sub-Domain Enumeration is Not Supported for IP Addresses, exiting")
        sys.exit(1)

    if wback:
        from modules.wayback import timetravel

        log_writer("Starting wayback enum...")
        timetravel(hostname, data, out_settings)

    if pscan:
        from modules.portscan import scan

        log_writer("Starting port scan...")
        scan(ip, out_settings, data, threads)

    if dirrec:
        from modules.dirrec import hammer

        log_writer("Starting dir enum...")
        hammer(target, threads, tout, wdlist, redir, sslv, out_settings, data, filext)

    if not any(
        [full, headinfo, sslinfo, whois, crawl, dns, subd, wback, pscan, dirrec]
    ):
        print(f"\n{R}[-]{W} Error : At least One Argument is Required with URL")
        log_writer("At least One Argument is Required with URL, exiting")
        output = "None"
        sys.exit(1)

    end_time = datetime.datetime.now() - start_time
    total_seconds = int(end_time.total_seconds())
    hours, remainder = divmod(total_seconds, 3600)
    minutes, seconds = divmod(remainder, 60)
    if hours:
        readable = f"{hours}h {minutes}m {seconds}s"
    elif minutes:
        readable = f"{minutes}m {seconds}s"
    else:
        readable = f"{seconds}s"
    print(f"\n{G}[+]{W} Completed in {readable}\n")
    log_writer(f"Completed in {readable}")
    print(f"{G}[+]{W} Exported : {respath}")
    log_writer(f"Exported to {respath}")
    log_writer(f"{'-' * 30}")
    sys.exit()
except KeyboardInterrupt:
    print(f"{R}[-]{W} Keyboard Interrupt.\n")
    log_writer("Keyboard interrupt, exiting")
    log_writer(f"{'-' * 30}")
    sys.exit(130)



#!/usr/bin/env python3

import asyncio
from json import loads

from modules.write_log import log_writer

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow


async def certspot(hostname, session):
    subdomains = []
    error = None

    url = "https://api.certspotter.com/v1/issuances"
    cs_params = {
        "domain": hostname,
        "expand": "dns_names",
        "include_subdomains": "true",
    }

    try:
        async with session.get(url, params=cs_params) as resp:
            status = resp.status
            if status == 200:
                json_data = await resp.text()
                json_read = loads(json_data)
                for i in range(0, len(json_read)):
                    domains = json_read[i]["dns_names"]
                    subdomains.extend(domains)
            else:
                error = f"Status : {status}"
                log_writer(f"[certspot_subs] Status = {status}, expected 200")
    except asyncio.TimeoutError:
        error = "Request Timeout"
        log_writer(f"[certspot_subs] Exception = {error}")
    except Exception as exc:
        error = exc
        log_writer(f"[certspot_subs] Exception = {exc}")
    log_writer("[certspot_subs] Completed")
    return "CertSpotter", subdomains, error



#!/usr/bin/env python3

import socket
import ssl
from datetime import timezone

from cryptography import x509
from cryptography.hazmat.backends import default_backend

from modules.export import export
from modules.write_log import log_writer

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow
HEADER = "\033[1;35m"  # bold magenta


def cert(hostname, sslp, output, data):
    result = {}
    presence = False
    print(f"\n{HEADER}━━━ TLS/SSL {'━' * 30}{W}\n")

    port_test = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    port_test.settimeout(5)
    try:
        port_test.connect((hostname, sslp))
        port_test.close()
        presence = True
    except Exception:
        port_test.close()
        print(f"{R}[-]{W} SSL is not Present on Target URL...Skipping...")
        result.update({"Error": "SSL is not Present on Target URL"})
        log_writer("[sslinfo] SSL is not Present on Target URL...Skipping...")

    def unpack(nested_tuple, pair):
        for item in nested_tuple:
            if isinstance(item, tuple):
                if len(item) == 2:
                    pair[item[0]] = item[1]
                else:
                    unpack(item, pair)
            else:
                pair[nested_tuple.index(item)] = item

    def process_cert(info):
        pair = {}
        for key, val in info.items():
            if isinstance(val, tuple):
                print(f"{G}{key}{W}")
                unpack(val, pair)
                for sub_key, sub_val in pair.items():
                    print(f"    └╴{sub_key} : {sub_val}")
                    result.update({f"{key}-{sub_key}": sub_val})
                pair.clear()
                print()
            elif isinstance(val, dict):
                print(f"{G}{key}{W}")
                for sub_key, sub_val in val.items():
                    print(f"    └╴{sub_key} : {sub_val}")
                    result.update({f"{key}-{sub_key}": sub_val})
                print()
            elif isinstance(val, list):
                print(f"{G}{key}{W}")
                for sub_val in val:
                    print(f"    └╴{val.index(sub_val)} : {sub_val}")
                    result.update({f"{key}-{val.index(sub_val)}": sub_val})
                print()
            else:
                print(f"{G}{key}{W} : {val}")
                result.update({key: val})
                print()

    if presence:
        ctx = ssl.create_default_context()
        ctx.check_hostname = False
        ctx.verify_mode = ssl.CERT_NONE
        sock = socket.socket()
        sock.settimeout(5)
        ssl_conn = ctx.wrap_socket(sock, server_hostname=hostname)
        ssl_conn.connect((hostname, sslp))
        x509_cert = ssl_conn.getpeercert(binary_form=True)
        decoded_cert = x509.load_der_x509_certificate(x509_cert, default_backend())

        subject_dict = {}
        issuer_dict = {}

        def name_to_dict(attribute):
            attr_name = attribute.oid._name
            attr_value = attribute.value
            return attr_name, attr_value

        for attribute in decoded_cert.subject:
            name, value = name_to_dict(attribute)
            subject_dict[name] = value

        for attribute in decoded_cert.issuer:
            name, value = name_to_dict(attribute)
            issuer_dict[name] = value

        # Handle `not_valid_before` and `not_valid_after` with compatibility
        if hasattr(decoded_cert, "not_valid_before_utc") and hasattr(
            decoded_cert, "not_valid_after_utc"
        ):
            not_before = decoded_cert.not_valid_before_utc
            not_after = decoded_cert.not_valid_after_utc
        else:
            # Fallback for older versions
            not_before = decoded_cert.not_valid_before.replace(tzinfo=timezone.utc)
            not_after = decoded_cert.not_valid_after.replace(tzinfo=timezone.utc)

        cert_dict = {
            "protocol": ssl_conn.version(),
            "cipher": ssl_conn.cipher(),
            "subject": subject_dict,
            "issuer": issuer_dict,
            "version": decoded_cert.version,
            "serialNumber": decoded_cert.serial_number,
            "notBefore": not_before.strftime("%b %d %H:%M:%S %Y GMT"),
            "notAfter": not_after.strftime("%b %d %H:%M:%S %Y GMT"),
        }

        extensions = decoded_cert.extensions
        for ext in extensions:
            if ext.oid != x509.ExtensionOID.SUBJECT_ALTERNATIVE_NAME:
                continue
            san_entries = ext.value
            subject_alt_names = []
            for entry in san_entries:
                if isinstance(entry, x509.DNSName):
                    subject_alt_names.append(entry.value)
            cert_dict["subjectAltName"] = subject_alt_names

        process_cert(cert_dict)
    result.update({"exported": False})

    if output:
        fname = f"{output['directory']}/ssl.{output['format']}"
        output["file"] = fname
        data["module-SSL Certificate Information"] = result
        export(output, data)
    log_writer("[sslinfo] Completed")



#!/usr/bin/env python3

import asyncio

import dns.asyncresolver

from modules.export import export
from modules.write_log import log_writer

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow
HEADER = "\033[1;35m"  # bold magenta


def dnsrec(domain, dns_servers, output, data):
    result = {}
    print(f"\n{HEADER}━━━ DNS Enum {'━' * 30}{W}\n")

    DNS_GROUPS = {
        "core": ["A", "AAAA", "CNAME", "PTR", "NS", "SOA", "MX"],
        "security": [
            "CAA",
            "DNSKEY",
            "CDNSKEY",
            "CDS",
            "DS",
            "RRSIG",
            "NSEC",
            "NSEC3",
            "NSEC3PARAM",
            "TLSA",
            "SSHFP",
            "SMIMEA",
            "OPENPGPKEY",
            "TA",
            "DLV",
            "SIG",
            "KEY",
        ],
        "mail": ["TXT", "SRV", "NAPTR", "URI"],
        "network": ["IPSECKEY", "HIP", "LOC", "SVCB", "HTTPS", "APL", "AFSDB", "KX"],
        "zone": ["DNAME", "CSYNC", "ZONEMD", "TKEY", "TSIG"],
        "info": ["HINFO", "RP", "CERT", "DHCID", "EUI48", "EUI64", "HINFO"],
    }

    full_ans = []

    res = dns.asyncresolver.Resolver(configure=False)
    res.nameservers = [sv.strip() for sv in dns_servers.split(",")]

    async def fetch_records(res, domain, record):
        answer = await res.resolve(domain, record)
        return answer

    for dns_grp in DNS_GROUPS:
        group_printed = False
        for dns_record in DNS_GROUPS.get(dns_grp):
            try:
                ans = asyncio.run(fetch_records(res, domain, dns_record))
                for record_data in ans:
                    full_ans.append(f"{dns_record} : {record_data.to_text()}")
                    if not group_printed:
                        print()
                        group_printed = True
                    if dns_grp == "security":
                        print(f"{Y}{dns_record.ljust(10)}{W} : {record_data.to_text()}")
                    else:
                        print(f"{G}{dns_record.ljust(10)}{W} : {record_data.to_text()}")
            except dns.resolver.NoAnswer as exc:
                log_writer(f"[dns] Exception = {exc}")
            except dns.resolver.NoMetaqueries as exc:
                log_writer(f"[dns] Exception = {exc}")
            except dns.resolver.NoNameservers as exc:
                log_writer(f"[dns] Exception = {exc}")
            except dns.resolver.NXDOMAIN as exc:
                log_writer(f"[dns] Exception = {exc}")
                print(f"{R}[-]{W} DNS Records Not Found!")
                if output != "None":
                    result.setdefault("dns", ["DNS Records Not Found"])
                return

    for entry in full_ans:
        if output != "None":
            result.setdefault("dns", []).append(entry)

    dmarc_target = f"_dmarc.{domain}"
    try:
        dmarc_ans = asyncio.run(fetch_records(res, dmarc_target, "TXT"))
        for entry in dmarc_ans:
            print(f"{Y}{'DMARC'.ljust(10)}{W} : {entry.to_text()}")
            if output != "None":
                result.setdefault("dmarc", []).append(f"DMARC : {entry.to_text()}")
    except dns.resolver.NoAnswer as exc:
        log_writer(f"[dns.dmarc] Exception = {exc}")
    except dns.resolver.NoMetaqueries as exc:
        log_writer(f"[dns.dmarc] Exception = {exc}")
    except dns.resolver.NoNameservers as exc:
        log_writer(f"[dns.dmarc] Exception = {exc}")
    except dns.resolver.NXDOMAIN as exc:
        log_writer(f"[dns.dmarc] Exception = {exc}")
        print(f"\n{R}[-] {C}DMARC Record Not Found!{W}")
        if output != "None":
            result.setdefault("dmarc", ["DMARC Record Not Found!"])

    result.update({"exported": False})

    if output != "None":
        data["module-DNS Enumeration"] = result
        fname = f"{output['directory']}/dns_records.{output['format']}"
        output["file"] = fname
        export(output, data)
    log_writer("[dns] Completed")



#!/usr/bin/env python3
import asyncio
from json import loads

from modules.write_log import log_writer

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow


async def urlscan(hostname, session):
    subdomains = []
    error = None

    url = f"https://urlscan.io/api/v1/search/?q=domain:{hostname}"
    try:
        async with session.get(url) as resp:
            status = resp.status
            if status == 200:
                output = await resp.text()
                json_data = loads(output)["results"]
                for entry in json_data:
                    subdomains.append(entry["task"]["domain"])
            else:
                error = f"Status : {status}"
                log_writer(f"[urlscan_subs] Status = {status}, expected 200")
    except asyncio.TimeoutError:
        error = "Request Timeout"
        log_writer(f"[urlscan_subs] Exception = {error}")
    except Exception as exc:
        print(f"{R}[-] {C}UrlScan Exception : {W}{exc}")
    log_writer("[urlscan_subs] Completed")

    return "UrlScan", subdomains, error



#!/usr/bin/env python3

import asyncio

from modules.write_log import log_writer

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow


async def zoomeye(hostname, api_key, session):
    subdomains = []
    error = None

    if api_key is not None:
        url = f"https://api.zoomeye.ai/domain/search?q={hostname}&type=0"
        header = {"API-KEY": api_key, "User-Agent": "curl"}

        try:
            async with session.get(url, headers=header) as resp:
                status = resp.status
                if status == 200:
                    json_data = await resp.json()
                    subdomain_list = json_data["list"]
                    subdomains = [subd["name"] for subd in subdomain_list]
                else:
                    error = f"Status : {status}"
                    log_writer(f"[zoomeye_subs] Status = {status}, expected 200")
        except asyncio.TimeoutError:
            error = "Request Timeout"
            log_writer(f"[zoomeye_subs] Exception = {error}")
        except Exception as exc:
            print(f"{R}[-] {C}zoomeye Exception : {W}{exc}")
            log_writer(f"[zoomeye_subs] Exception = {exc}")
    else:
        error = "API key not configured"
        log_writer(f"[zoomeye_subs] {error}")
    log_writer("[zoomeye_subs] Completed")

    return "ZoomEye", subdomains, error



#!/usr/bin/env python3

import asyncio
import re

from modules.write_log import log_writer

R = "\033[31m"
G = "\033[32m"
C = "\033[36m"
W = "\033[0m"
Y = "\033[33m"


async def github(hostname, api_key, session):
    subdomains = []
    error = None

    if api_key is not None:
        header = {
            "Authorization": f"Bearer {api_key}",
            "Accept": "application/vnd.github.text-match+json",
            "X-GitHub-Api-Version": "2022-11-28",
        }
        params = {
            "q": hostname,
            "per_page": 100,
        }
        pattern = re.compile(rf"(?:[a-zA-Z0-9_*-]+\.)+{re.escape(hostname)}")

        try:
            for page in range(1, 10):
                params = {"q": hostname, "per_page": 100, "page": page}
                async with session.get(
                    "https://api.github.com/search/code",
                    headers=header,
                    params=params,
                ) as resp:
                    status = resp.status
                    if status == 403:
                        error = "Rate limited"
                        log_writer("[github_subs] Rate limited")
                        break
                    if status != 200:
                        error = f"Status = {status}"
                        log_writer(f"[github_subs] Status = {status}, expected 200")
                        break
                    json_data = await resp.json()
                    items = json_data.get("items", [])
                    if not items:
                        break
                    for item in items:
                        for match in item.get("text_matches", []):
                            matches = pattern.findall(match.get("fragment", ""))
                            subdomains.extend(matches)
                            subdomains = [
                                s
                                for s in subdomains
                                if not re.search(r"(?:^|\.)[0-9A-Fa-f]{2}[A-Z]", s)
                            ]
                await asyncio.sleep(0.1)
        except asyncio.TimeoutError:
            error = "Request Timeout"
            log_writer(f"[github_subs] Exception = {error}")
        except Exception as exc:
            error = str(exc)
            log_writer(f"[github_subs] Exception = {exc}")
    else:
        error = "API key not configured"
        log_writer(f"[github_subs] {error}")

    log_writer("[github_subs] Completed")
    return "GitHub", subdomains, error



#!/usr/bin/env python3

from os import environ
from json import loads, dumps
import modules.subdom as parent
from modules.write_log import log_writer

R = '\033[31m'  # red
G = '\033[32m'  # green
C = '\033[36m'  # cyan
W = '\033[0m'   # white
Y = '\033[33m'  # yellow


async def binedge(hostname, conf_path, session):
    binedge_key = environ.get('FR_BINEDGE_KEY')

    if not binedge_key:
        log_writer('[binedge_subs] key missing in env')
        with open(f'{conf_path}/keys.json', 'r') as keyfile:
            json_read = keyfile.read()

        json_load = loads(json_read)
        try:
            binedge_key = json_load['binedge']
        except KeyError:
            log_writer('[binedge_subs] key missing in keys.json')
            with open(f'{conf_path}/keys.json', 'w') as outfile:
                json_load['binedge'] = None
                binedge_key = None
                outfile.write(
                    dumps(json_load, sort_keys=True, indent=4)
                )

    if binedge_key is not None:
        print(f'{Y}[!] {C}Requesting {G}BinaryEdge{W}')
        url = f'https://api.binaryedge.io/v2/query/domains/subdomain/{hostname}'
        header = {'X-key': binedge_key}

        try:
            async with session.get(url, headers=header) as resp:
                status = resp.status
                if status == 200:
                    json_data = await resp.json()
                    subdomains = json_data['events']
                    print(f'{G}[+] {Y}binedge {W}found {C}{len(subdomains)} {W}subdomains!')
                    parent.found.extend(subdomains)
                else:
                    print(f'{R}[-] {C}binedge Status : {W}{status}')
                    log_writer(f'[binedge_subs] Status = {status}, expected 200')

        except Exception as exc:
            print(f'{R}[-] {C}binedge Exception : {W}{exc}')
            log_writer(f'[binedge_subs] Exception = {exc}')
    else:
        print(f'{Y}[!] Skipping binedge : {W}API key not found!')
        log_writer('[binedge_subs] API key not found')
    log_writer('[binedge_subs] Completed')



#!/usr/bin/env python3

import asyncio
import re
import threading

import bs4
import requests
import tldextract

from modules.export import export
from modules.write_log import log_writer

requests.packages.urllib3.disable_warnings()
import settings as config

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow
HEADER = "\033[1;35m"  # bold magenta

user_agent = {"User-Agent": f"FinalRecon/{config.version}"}


def crawler(target, protocol, netloc, output, data):
    r_total = []
    sm_total = []
    css_total = []
    js_total = []
    int_total = []
    ext_total = []
    img_total = []
    sm_crawl_total = []
    js_crawl_total = []
    total = []

    print(f"\n{HEADER}━━━ Crawler {'━' * 30}{W}\n")

    try:
        rqst = requests.get(target, headers=user_agent, verify=False, timeout=10)
    except Exception as exc:
        print(f"{R}[-]{W} Exception : {exc}")
        log_writer(f"[crawler] Exception = {exc}")
        return

    status = rqst.status_code
    if status == 200:
        page = rqst.content
        soup = bs4.BeautifulSoup(page, "lxml")
        r_url = f"{protocol}://{netloc}/robots.txt"
        sm_url = f"{protocol}://{netloc}/sitemap.xml"
        base_url = f"{protocol}://{netloc}"
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        tasks = asyncio.gather(
            robots(r_url, r_total, sm_total, base_url, data, output),
            sitemap(sm_url, sm_total, data, output),
            css(target, css_total, data, soup, output),
            js_scan(target, js_total, data, soup, output),
            internal_links(target, int_total, data, soup, output),
            external_links(target, ext_total, data, soup, output),
            images(target, img_total, data, soup, output),
            sm_crawl(data, sm_crawl_total, sm_total, sm_url, output),
            js_crawl(data, js_crawl_total, js_total, output),
        )
        loop.run_until_complete(tasks)
        loop.close()
        stats(
            output,
            r_total,
            sm_total,
            css_total,
            js_total,
            int_total,
            ext_total,
            img_total,
            sm_crawl_total,
            js_crawl_total,
            total,
            data,
            soup,
        )
        log_writer("[crawler] Completed")
    else:
        print(f"{R}[-]{W} Status : {status}")
        log_writer(f"[crawler] Status code = {status}, expected 200")


def url_filter(target, link):
    if all([link.startswith("/") is True, link.startswith("//") is False]):
        ret_url = target + link
        return ret_url

    if link.startswith("//") is True:
        ret_url = link.replace("//", "http://")
        return ret_url

    if all(
        [
            link.find("//") == -1,
            link.find("../") == -1,
            link.find("./") == -1,
            link.find("http://") == -1,
            link.find("https://") == -1,
        ]
    ):
        ret_url = f"{target}/{link}"
        return ret_url

    if all([link.find("http://") == -1, link.find("https://") == -1]):
        ret_url = link.replace("//", "http://")
        ret_url = link.replace("../", f"{target}/")
        ret_url = link.replace("./", f"{target}/")
        return ret_url
    return link


async def robots(robo_url, r_total, sm_total, base_url, data, output):
    print(f"{C}[*]{W} Looking for robots.txt", end="", flush=True)

    try:
        r_rqst = requests.get(robo_url, headers=user_agent, verify=False, timeout=10)
        r_sc = r_rqst.status_code
        if r_sc == 200:
            print(f"{G}{'['.rjust(9, '.')} Found ]{W}")
            print(f"{C}[*]{W} Extracting robots Links", end="", flush=True)
            r_page = r_rqst.text
            r_scrape = r_page.split("\n")
            for entry in r_scrape:
                if any(
                    [
                        entry.find("Disallow") == 0,
                        entry.find("Allow") == 0,
                        entry.find("Sitemap") == 0,
                    ]
                ):
                    url = entry.split(": ", 1)[1].strip()
                    tmp_url = url_filter(base_url, url)

                    if tmp_url is not None:
                        r_total.append(url_filter(base_url, url))

                    if url.endswith("xml"):
                        sm_total.append(url)

            r_total = set(r_total)
            print(f"{G}{'['.rjust(8, '.')} {len(r_total)} ]")
            exporter(data, output, r_total, "robots")

        elif r_sc == 404:
            print(f"{R}{'['.rjust(9, '.')} Not Found ]{W}")

        else:
            print(f"{R}{'['.rjust(9, '.')} {r_sc} ]{W}")

    except Exception as exc:
        print(f"\n{R}[-]{W} Exception : {exc}")
        log_writer(f"[crawler.robots] Exception = {exc}")


async def sitemap(target_url, sm_total, data, output):
    print(f"{C}[*]{W} Looking for sitemap.xml", end="", flush=True)
    try:
        sm_rqst = requests.get(target_url, headers=user_agent, verify=False, timeout=10)
        sm_sc = sm_rqst.status_code
        if sm_sc == 200:
            print(f"{G}{'['.rjust(8, '.')} Found ]{W}")
            print(f"{C}[*]{W} Extracting sitemap Links", end="", flush=True)
            sm_page = sm_rqst.content
            sm_soup = bs4.BeautifulSoup(sm_page, "xml")
            links = sm_soup.find_all("loc")
            for url in links:
                url = url.get_text()
                if url is not None:
                    sm_total.append(url)

            sm_total = set(sm_total)
            print(f"{G}{'['.rjust(7, '.')} {len(sm_total)} ]{W}")
            exporter(data, output, sm_total, "sitemap")
        elif sm_sc == 404:
            print(f"{R}{'['.rjust(8, '.')} Not Found ]{W}")
        else:
            print(f"{R}{'['.rjust(8, '.')} Status Code : {sm_sc} ]{W}")
    except Exception as exc:
        print(f"\n{R}[-]{W} Exception : {exc}")
        log_writer(f"[crawler.sitemap] Exception = {exc}")


async def css(target, css_total, data, soup, output):
    print(f"{C}[*]{W} Extracting CSS Links", end="", flush=True)
    css_links = soup.find_all("link", href=True)

    for link in css_links:
        url = link.get("href")
        if url is not None and ".css" in url:
            css_total.append(url_filter(target, url))

    css_total = set(css_total)
    print(f"{G}{'['.rjust(11, '.')} {len(css_total)} ]{W}")
    exporter(data, output, css_total, "css")


async def js_scan(target, js_total, data, soup, output):
    print(f"{C}[*]{W} Extracting Javascript Links", end="", flush=True)
    scr_tags = soup.find_all("script", src=True)

    for link in scr_tags:
        url = link.get("src")
        if url is not None and ".js" in url:
            tmp_url = url_filter(target, url)
            if tmp_url is not None:
                js_total.append(tmp_url)

    js_total = set(js_total)
    print(f"{G}{'['.rjust(4, '.')} {len(js_total)} ]{W}")
    exporter(data, output, js_total, "javascripts")


async def internal_links(target, int_total, data, soup, output):
    print(f"{C}[*]{W} Extracting Internal Links", end="", flush=True)

    ext = tldextract.extract(target)
    domain = ext.registered_domain

    links = soup.find_all("a")
    for link in links:
        url = link.get("href")
        if url is not None:
            if domain in url:
                int_total.append(url)

    int_total = set(int_total)
    print(f"{G}{'['.rjust(6, '.')} {len(int_total)} ]{W}")
    exporter(data, output, int_total, "internal_urls")


async def external_links(target, ext_total, data, soup, output):
    print(f"{C}[*]{W} Extracting External Links", end="", flush=True)

    ext = tldextract.extract(target)
    domain = ext.registered_domain

    links = soup.find_all("a")
    for link in links:
        url = link.get("href")
        if url is not None:
            if domain not in url and "http" in url:
                ext_total.append(url)

    ext_total = set(ext_total)
    print(f"{G}{'['.rjust(6, '.')} {len(ext_total)} ]{W}")
    exporter(data, output, ext_total, "external_urls")


async def images(target, img_total, data, soup, output):
    print(f"{C}[*]{W} Extracting Images", end="", flush=True)
    image_tags = soup.find_all("img")

    for link in image_tags:
        url = link.get("src")
        if url is not None and len(url) > 1:
            img_total.append(url_filter(target, url))

    img_total = set(img_total)
    print(f"{G}{'['.rjust(14, '.')} {len(img_total)} ]{W}")
    exporter(data, output, img_total, "images")


async def sm_crawl(data, sm_crawl_total, sm_total, sm_url, output):
    print(f"{C}[*]{W} Crawling Sitemaps", end="", flush=True)

    threads = []

    def fetch(site_url):
        try:
            sm_rqst = requests.get(
                site_url, headers=user_agent, verify=False, timeout=10
            )
            sm_sc = sm_rqst.status_code
            if sm_sc == 200:
                sm_data = sm_rqst.content.decode()
                sm_soup = bs4.BeautifulSoup(sm_data, "xml")
                links = sm_soup.find_all("loc")
                for url in links:
                    url = url.get_text()
                    if url is not None:
                        sm_crawl_total.append(url)
            elif sm_sc == 404:
                # print(R + '['.rjust(8, '.') + ' Not Found ]' + W)
                pass
            else:
                # print(R + '['.rjust(8, '.') + ' {} ]'.format(sm_sc) + W)
                pass
        except Exception as exc:
            # print(f'\n{R}[-] Exception : {C}{exc}{W}')
            log_writer(f"[crawler.sm_crawl] Exception = {exc}")

    for site_url in sm_total:
        if site_url != sm_url:
            if site_url.endswith("xml") is True:
                task = threading.Thread(target=fetch, args=[site_url])
                task.daemon = True
                threads.append(task)
                task.start()

    for thread in threads:
        thread.join()

    sm_crawl_total = set(sm_crawl_total)
    print(f"{G}{'['.rjust(14, '.')} {len(sm_crawl_total)} ]{W}")
    exporter(data, output, sm_crawl_total, "urls_inside_sitemap")


async def js_crawl(data, js_crawl_total, js_total, output):
    print(f"{C}[*]{W} Crawling Javascripts", end="", flush=True)

    threads = []

    def fetch(js_url):
        try:
            js_rqst = requests.get(js_url, headers=user_agent, verify=False, timeout=10)
            js_sc = js_rqst.status_code
            if js_sc == 200:
                js_data = js_rqst.content.decode()
                js_data = js_data.split(";")
                for line in js_data:
                    if any(["http://" in line, "https://" in line]):
                        found = re.findall(r"\"(http[s]?://.*?)\"", line)
                        for item in found:
                            if len(item) > 8:
                                js_crawl_total.append(item)
        except Exception as exc:
            # print(f'\n{R}[-] Exception : {C}{exc}{W}')
            log_writer(f"[crawler.js_crawl] Exception = {exc}")

    for js_url in js_total:
        task = threading.Thread(target=fetch, args=[js_url])
        task.daemon = True
        threads.append(task)
        task.start()

    for thread in threads:
        thread.join()

    js_crawl_total = set(js_crawl_total)
    print(f"{G}{'['.rjust(11, '.')} {len(js_crawl_total)} ]{W}")
    exporter(data, output, js_crawl_total, "urls_inside_js")


def exporter(data, output, list_name, file_name):
    data[f"module-crawler-{file_name}"] = {"links": list(list_name)}
    data[f"module-crawler-{file_name}"].update({"exported": False})
    fname = f"{output['directory']}/{file_name}.{output['format']}"
    output["file"] = fname
    export(output, data)


def stats(
    output,
    r_total,
    sm_total,
    css_total,
    js_total,
    int_total,
    ext_total,
    img_total,
    sm_crawl_total,
    js_crawl_total,
    total,
    data,
    soup,
):
    total.extend(r_total)
    total.extend(sm_total)
    total.extend(css_total)
    total.extend(js_total)
    total.extend(js_crawl_total)
    total.extend(sm_crawl_total)
    total.extend(int_total)
    total.extend(ext_total)
    total.extend(img_total)
    total = set(total)

    print(f"\n{G}[+]{W} Total Unique Links Extracted : {len(total)}")

    if output != "None":
        if len(total) != 0:
            data["module-crawler-stats"] = {
                "Total Unique Links Extracted": str(len(total))
            }
            try:
                target_title = soup.title.string
            except AttributeError:
                target_title = "None"
            data["module-crawler-stats"].update({"Title ": str(target_title)})

            data["module-crawler-stats"].update(
                {
                    "total_urls_robots": len(r_total),
                    "total_urls_sitemap": len(sm_total),
                    "total_urls_css": len(css_total),
                    "total_urls_js": len(js_total),
                    "total_urls_in_js": len(js_crawl_total),
                    "total_urls_in_sitemaps": len(sm_crawl_total),
                    "total_urls_internal": len(int_total),
                    "total_urls_external": len(ext_total),
                    "total_urls_images": len(img_total),
                    "total_urls": len(total),
                }
            )
            data["module-crawler-stats"].update({"exported": False})



import logging
import settings


def log_writer(message):
    logging.basicConfig(
        filename=settings.log_file_path,
        encoding='utf-8',
        level=logging.INFO,
        format='[%(asctime)s] : %(message)s',
        datefmt='%m/%d/%Y %I:%M:%S %p'
    )
    logging.info(message)



#!/usr/bin/env python3

import asyncio
from json import loads

from modules.write_log import log_writer

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow


async def anubisdb(hostname, session):
    subdomains = []
    error = None
    url = f"https://anubisdb.com/anubis/subdomains/{hostname}"
    try:
        async with session.get(url) as resp:
            status = resp.status
            if status == 200:
                output = await resp.text()
                subdomains.extend(loads(output))
            elif status == 300:
                error = f"Status : {status}, no subdomains found"
                log_writer(f"[anubis_subs] Status = {status}, no subdomains found")
            else:
                error = f"Status : {status}"
                log_writer(f"[anubis_subs] Status = {status}, expected 200")
                log_writer(f"[anubis_subs] Exception = {await resp.text()}")
    except asyncio.TimeoutError:
        error = "Request Timeout"
    except Exception as exc:
        error = exc
    log_writer(f"[anubis_subs] Exception = {error}")
    log_writer("[anubis_subs] Completed")

    return "AnubisDB", subdomains, error



#!/usr/bin/env python3

import asyncio
from datetime import datetime, timezone
from json import load

from modules.export import export
from modules.write_log import log_writer

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow
HEADER = "\033[1;35m"  # bold magenta


async def get_whois(domain, server):
    whois_result = {}
    reader, writer = await asyncio.open_connection(server, 43)
    writer.write((domain + "\r\n").encode())

    raw_resp = b""
    while True:
        chunk = await reader.read(4096)
        if not chunk:
            break
        raw_resp += chunk

    writer.close()
    await writer.wait_closed()
    raw_result = raw_resp.decode()

    if "No match for" in raw_result:
        whois_result = None

    res_parts = raw_result.split(">>>", 1)
    whois_result["whois"] = res_parts[0]
    return whois_result


def parse_whois(raw):
    IMPORTANT = {
        "Domain Name",
        "Registrar",
        "Creation Date",
        "Updated Date",
        "Registry Expiry Date",
        "Domain Status",
        "Name Server",
        "DNSSEC",
    }

    for line in raw.splitlines():
        color = W
        suffix = ""
        line = line.strip()
        if ":" not in line:
            continue
        key, _, val = line.partition(":")
        key, val = key.strip(), val.strip()
        if not key or not val:
            continue
        if key not in IMPORTANT:
            continue
        if key == "Registry Expiry Date":
            try:
                expiry = datetime.fromisoformat(val.replace("Z", "+00:00"))
                days = (expiry - datetime.now(timezone.utc)).days
                color = R if days < 30 else Y if days < 90 else G
                suffix = f"({days} days)"
            except Exception:
                pass
        elif key == "DNSSEC" and val.lower() == "unsigned":
            color = R
        elif key == "Domain Status":
            val = val.split("https://")[0].strip()
        print(f"{C}{key.ljust(30)}{W} : {color}{val} {suffix}{W}")


def whois_lookup(domain, tld, script_path, output, data):
    result = {}
    db_path = f"{script_path}/whois_servers.json"
    with open(db_path, "r") as db_file:
        db_json = load(db_file)
    print(f"\n{HEADER}━━━ WHOIS {'━' * 30}{W}\n")

    try:
        whois_sv = db_json[tld]
        whois_info = asyncio.run(get_whois(f"{domain}.{tld}", whois_sv))
        parse_whois(whois_info["whois"])
        result.update(whois_info)
    except KeyError:
        print(f"{R}[-] Error : {C}This domain suffix is not supported.{W}")
        result.update({"Error": "This domain suffix is not supported."})
        log_writer("[whois] Exception = This domain suffix is not supported.")
    except Exception as exc:
        print(f"{R}[-] Error : {C}{exc}{W}")
        result.update({"Error": str(exc)})
        log_writer(f"[whois] Exception = {exc}")

    result.update({"exported": False})

    if output != "None":
        fname = f"{output['directory']}/whois.{output['format']}"
        output["file"] = fname
        data["module-whois"] = result
        export(output, data)
    log_writer("[whois] Completed")



#!/usr/bin/env python3

from os import environ
from json import loads, dumps
import modules.subdom as parent
from modules.write_log import log_writer

R = '\033[31m'  # red
G = '\033[32m'  # green
C = '\033[36m'  # cyan
W = '\033[0m'   # white
Y = '\033[33m'  # yellow


async def fb_cert(hostname, conf_path, session):
	fb_key = environ.get('FR_FB_KEY')

	if not fb_key:
		log_writer('[fb_subs] key missing in env')
		with open(f'{conf_path}/keys.json', 'r') as keyfile:
			json_read = keyfile.read()

		json_load = loads(json_read)
		try:
			fb_key = json_load['facebook']
		except KeyError:
			log_writer('[fb_subs] key missing in keys.json')
			with open(f'{conf_path}/keys.json', 'w') as outfile:
				json_load['facebook'] = None
				fb_key = None
				outfile.write(
					dumps(json_load, sort_keys=True, indent=4)
				)

	if fb_key is not None:
		print(f'{Y}[!] {C}Requesting {G}Facebook{W}')
		url = 'https://graph.facebook.com/certificates'
		fb_params = {
			'query': hostname,
			'fields': 'domains',
			'access_token': fb_key
		}
		try:
			async with session.get(url, params=fb_params) as resp:
				status = resp.status
				if status == 200:
					json_data = await resp.text()
					json_read = loads(json_data)
					domains = json_read['data']
					print(f'{G}[+] {Y}Facebook {W}found {C}{len(domains)} {W}subdomains!')
					for i in range(0, len(domains)):
						parent.found.extend(json_read['data'][i]['domains'])
				else:
					print(f'{R}[-] {C}Facebook Status : {W}{status}')
					log_writer(f'[fb_subs] Status = {status}, expected 200')
		except Exception as exc:
			print(f'{R}[-] {C}Facebook Exception : {W}{exc}')
			log_writer(f'[fb_subs] Exception = {exc}')
	else:
		print(f'{Y}[!] Skipping Facebook : {W}API key not found!')
		log_writer('[fb_subs] API key not found')
	log_writer('[fb_subs] Completed')



#!/usr/bin/env python3

import asyncio

from modules.write_log import log_writer

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow


async def bevigil(hostname, api_key, session):
    subdomains = []
    error = None

    if api_key is not None:
        url = f"https://osint.bevigil.com/api/{hostname}/subdomains/"
        header = {"X-Access-Token": api_key}

        try:
            async with session.get(url, headers=header) as resp:
                status = resp.status
                if status == 200:
                    json_data: list = await resp.json()
                    subdomains = json_data.get("subdomains")
                else:
                    error = f"Status : {status}"
                    log_writer(f"[bevigil_subs] Status = {status}, expected 200")
        except asyncio.TimeoutError:
            error = "Request Timeout"
            log_writer(f"[bevigil_subs] Exception = {error}")
        except Exception as exc:
            error = exc
            log_writer(f"[bevigil_subs] Exception = {exc}")
    else:
        error = "API key not configured"
        log_writer("[bevigil_subs] API key not found")
    log_writer("[bevigil_subs] Completed")

    return "BeVigil", subdomains, error



#!/usr/bin/env python3

import re
from collections import defaultdict
from operator import itemgetter
from urllib.parse import parse_qs, urlparse

import requests
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

from modules.export import export
from modules.write_log import log_writer

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow
HEADER = "\033[1;35m"  # bold magenta

JUICY_EXT = {
    "js",
    "json",
    "xml",
    "yaml",
    "yml",
    "env",
    "config",
    "conf",
    "bak",
    "backup",
    "old",
    "sql",
    "db",
    "log",
    "txt",
    "zip",
    "tar",
    "gz",
    "key",
    "pem",
    "php",
    "asp",
    "aspx",
    "jsp",
}

JUICY_PATHS = re.compile(
    r"/(admin|api|v\d+|graphql|internal|debug|swagger|openapi|"
    r"metrics|actuator|\.git|\.env|backup|upload|export|"
    r"config|dashboard|console)(/|$|\?)",
    re.IGNORECASE,
)

JUICY_PARAMS = re.compile(
    r"[?&](url|redirect|next|return|goto|dest|file|path|"
    r"id|uid|user_id|account|order|invoice|"  # IDOR
    r"cmd|exec|command|query|search|"  # injection
    r"token|key|secret|api_key|access_token|auth|"  # secrets
    r"src|source|host|domain|endpoint|"  # SSRF
    r"callback|jsonp|"  # JSONP/open redirect
    r"template|include|page|view|load|"  # LFI/path traversal
    r"debug|test|admin)=",  # logic flaws
    re.IGNORECASE,
)


def analyze(url_list):
    intel = {
        "all_urls": [],
        "js_files": [],
        "api_endpoints": [],
        "juicy_paths": [],
        "juicy_params": [],
        "juicy_ext": defaultdict(list),
        "param_summary": defaultdict(int),
    }

    for url in url_list:
        if not url or not url.startswith("http"):
            continue

        intel["all_urls"].append(url)

        try:
            parsed = urlparse(url)
        except Exception:
            continue

        path = parsed.path.lower()
        ext = path.rsplit(".", 1)[-1] if "." in path.split("/")[-1] else ""

        if ext in JUICY_EXT:
            intel["juicy_ext"][ext].append(url)
        if ext == "js":
            intel["js_files"].append(url)
        if re.search(
            r"/api/|/graphql|/graphiql|/gql|/rest/|/v\d+|/rpc|/endpoint", path
        ):
            intel["api_endpoints"].append(url)
        elif JUICY_PATHS.search(parsed.path):
            intel["juicy_paths"].append(url)
        if parsed.query and JUICY_PARAMS.search("?" + parsed.query):
            intel["juicy_params"].append(url)
            for param in parse_qs(parsed.query):
                intel["param_summary"][param] += 1

    intel["param_summary"] = dict(
        sorted(intel["param_summary"].items(), key=itemgetter(1), reverse=True)
    )
    intel["juicy_ext"] = {k: list(set(v)) for k, v in intel["juicy_ext"].items()}
    return intel


def print_summary(intel):
    print(f" {G}❯{W} Total URLs         : {len(intel['all_urls'])}")
    print(f" {G}❯{W} JS files           : {len(intel['js_files'])}")
    print(f" {G}❯{W} API endpoints      : {len(intel['api_endpoints'])}")
    print(f" {G}❯{W} Interesting paths  : {len(intel['juicy_paths'])}")
    print(f" {G}❯{W} Interesting params : {len(intel['juicy_params'])}")

    if intel["juicy_ext"]:
        exts = ", ".join(
            f"{e}({len(v)})" for e, v in sorted(intel["juicy_ext"].items())
        )
        print(f" {Y}❯{W} Sensitive exts     : {exts}")

    if intel["param_summary"]:
        top = list(intel["param_summary"].items())[:8]
        print(f" {Y}❯{W} Top params         : {', '.join(f'{p}={c}' for p, c in top)}")

    if intel["juicy_ext"]:
        for ext, urls in sorted(intel["juicy_ext"].items()):
            print(f"\n{Y}.{ext} :{W}")
            for url in urls[:3]:
                print(url)

    if intel["api_endpoints"]:
        print(f"\n{Y}API Endpoints :{W}")
        for url in intel["api_endpoints"][:5]:
            print(url)

    if intel["juicy_params"]:
        print(f"\n{Y}Parameters :{W}")
        for url in intel["juicy_params"][:5]:
            print(url)


def timetravel(target, data, output):
    wayback_total = []
    result = {}
    r_data = None
    domain_query = f"{target}/*"

    print(f"\n{HEADER}━━━ Wayback Machine {'━' * 30}{W}\n")

    print(f"{C}[*]{W} Time travelling...")
    wm_url = "http://web.archive.org/cdx/search/cdx"

    payload = {
        "url": domain_query,
        "fl": "original",
        "collapse": "urlkey",
        "limit": 50000,
    }

    try:
        s = requests.Session()
        retry = Retry(total=3, backoff_factor=2, status_forcelist=[500, 502, 503])
        s.mount(
            "http://",
            HTTPAdapter(max_retries=retry),
        )

        rqst = s.get(wm_url, params=payload, timeout=30)
        r_sc = rqst.status_code
        if r_sc == 200:
            r_data = rqst.text
            r_data = set(r_data.split("\n"))
            wayback_total.extend(r_data)

            print(f"\n{C}[*]{W} Analyzing...\n")
            intel = analyze(r_data)
            print_summary(intel)
        else:
            print(f"\n{R}[-]{W} Status : {r_sc}")
            log_writer(f"[wayback] Status = {r_sc}, expected 200")

        if r_data:
            if output != "None":
                flat = []
                flat.append("\n=== JS FILES ===")
                flat.extend(intel["js_files"])
                flat.append("\n=== API ENDPOINTS ===")
                flat.extend(intel["api_endpoints"])
                flat.append("\n=== JUICY PARAMS ===")
                flat.extend(intel["juicy_params"])
                flat.append("\n=== SENSITIVE EXTENSIONS ===")
                for ext, urls in sorted(intel["juicy_ext"].items()):
                    flat.append(f".{ext}:")
                    flat.extend(urls)
                result["links"] = flat
                result.update({"exported": False})
                raw_results = {}
                raw_results.update({"wayback_urls": wayback_total})
                raw_results.update({"exported": False})
                data["module-wayback_urls"] = result
                fname = f"{output['directory']}/wayback_urls.{output['format']}"
                output["file"] = fname
                export(output, data)
                print(f"\n{C}[*]{W} Exported triaged urls at {fname}\n")
                data["module-wayback_urls-raw"] = raw_results
                fname_raw = f"{output['directory']}/wayback_urls_raw.{output['format']}"
                output["file"] = fname_raw
                export(output, data)
                print(f"{C}[*]{W} Exported raw urls at {fname_raw}")
        else:
            print(f"{R}[-]{W} No URLs Found!")
            log_writer("[wayback] No URLs Found!")
    except Exception as exc:
        print(f"\n{R}[-]{W} Exception : {exc}")
        log_writer(f"[wayback] Exception = {exc}")
    log_writer("[wayback] Completed")



#!/usr/bin/env python3

import asyncio
from json import loads

from modules.write_log import log_writer

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow


async def shodan(hostname, api_key, session):
    subdomains = []
    error = None

    if api_key is not None:
        url = f"https://api.shodan.io/dns/domain/{hostname}?key={api_key}"
        try:
            async with session.get(url) as resp:
                status = resp.status
                if status == 200:
                    json_data = await resp.text()
                    json_read = loads(json_data)
                    domains = json_read["subdomains"]
                    tmp_list = []
                    for i in range(0, len(domains)):
                        tmp_list.append(f"{domains[i]}.{hostname}")
                    subdomains.extend(tmp_list)
                else:
                    error = f"Status : {status}"
                    log_writer(f"[shodan_subs] Status = {status}, expected 200")
        except asyncio.TimeoutError:
            error = "Request Timeout"
            log_writer(f"[shodan_subs] Exception = {error}")
        except Exception as exc:
            error = exc
            log_writer(f"[shodan_subs] Exception = {exc}")
    else:
        error = "API key not configured"
        log_writer(f"[shodan_subs] {error}")
    log_writer("[shodan_subs] Completed")

    return "Shodan", subdomains, error



#!/usr/bin/env python3

from os import getenv, path, makedirs
from json import loads
from shutil import copytree

home = getenv('HOME')
usr_data = f'{home}/.local/share/finalrecon/dumps/'
conf_path = f'{home}/.config/finalrecon'
path_to_script = path.dirname(path.realpath(__file__))
src_conf_path = f'{path_to_script}/conf/'
meta_file_path = f'{path_to_script}/metadata.json'
keys_file_path = f'{conf_path}/keys.json'
conf_file_path = f'{conf_path}/config.json'
log_file_path = f'{home}/.local/share/finalrecon/run.log'

if not path.exists(conf_path):
    copytree(src_conf_path, conf_path, dirs_exist_ok=True)

if not path.exists(usr_data):
    makedirs(usr_data, exist_ok=True)

with open(meta_file_path) as meta_file:
    meta_read = meta_file.read()
    meta_json = loads(meta_read)

version = meta_json['version']

while True:
    with open(conf_file_path, 'r') as config_file:
        config_read = config_file.read()
        config_json = loads(config_read)

    try:
        timeout = config_json['common']['timeout']
        custom_dns = config_json['common']['dns_servers']
        ssl_port = config_json['ssl_cert']['ssl_port']
        port_scan_th = config_json['port_scan']['threads']
        dir_enum_th = config_json['dir_enum']['threads']
        dir_enum_redirect = config_json['dir_enum']['redirect']
        dir_enum_sslv = config_json['dir_enum']['verify_ssl']
        dir_enum_ext = config_json['dir_enum']['extension']
        dir_enum_wlist = f'{path_to_script}/wordlists/dirb_common.txt'
        export_fmt = config_json['export']['format']
    except KeyError:
        #  reset conf.json
        copytree(src_conf_path, conf_path, dirs_exist_ok=True)
    else:
        break



#!/usr/bin/env python3

import requests

from modules.export import export
from modules.write_log import log_writer

requests.packages.urllib3.disable_warnings()

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow
HEADER = "\033[1;35m"  # bold magenta

SECURITY_HEADERS = {
    "X-Frame-Options",
    "X-XSS-Protection",
    "Content-Security-Policy",
    "Content-Security-Policy-Report-Only",
    "Strict-Transport-Security",
    "X-Content-Type-Options",
    "Referrer-Policy",
    "Permissions-Policy",
}

REQUIRED_HEADERS = {
    "Strict-Transport-Security": "HSTS",
    "X-Content-Type-Options": "X-Content-Type-Options",
    "X-Frame-Options": "X-Frame-Options",
    "Content-Security-Policy": "CSP",
    "Referrer-Policy": "Referrer-Policy",
}

COOKIE_ATTRS = ["Secure", "HttpOnly", "SameSite", "Path", "Domain", "Expires"]


def headers(target, output, data):
    result = {}
    print(f"\n{HEADER}━━━ Headers {'━' * 30}{W}\n")
    try:
        rqst = requests.get(target, verify=False, timeout=10)
        hdrs = rqst.headers

        general = {
            k: v
            for k, v in hdrs.items()
            if k not in SECURITY_HEADERS and k != "Set-Cookie"
        }
        print(f"{G}❯ General{W}\n")
        for key, val in general.items():
            print(f"   {C}{key.ljust(30)}{W} : {val}")
            if output != "None":
                result[key] = val

        sec = {k: v for k, v in hdrs.items() if k in SECURITY_HEADERS}
        if sec:
            print(f"\n{Y}❯ Security{W}\n")
            for key, val in sec.items():
                print(f"  {Y}{key.ljust(30)}{W} : {val}")
                if output != "None":
                    result[key] = val

        missing = [
            label for header, label in REQUIRED_HEADERS.items() if header not in hdrs
        ]
        if missing:
            print(f"\n{R}❯ Missing{W}\n")
            for label in missing:
                print(f"   {R}{label}{W}")

        cookies = rqst.headers.get("Set-Cookie", "")
        if cookies:
            print(f"\n{Y}❯ Cookies{W}\n")
            for cookie in rqst.raw.headers.getlist("Set-Cookie"):
                parts = [p.strip() for p in cookie.split(";")]
                name = parts[0].split("=")[0]
                attrs = {
                    p.split("=")[0].strip(): p.split("=")[1].strip()
                    if "=" in p
                    else "✓"
                    for p in parts[1:]
                }
                secure = f"{G}✓{W}" if "Secure" in attrs else f"{R}✗{W}"
                httponly = f"{G}✓{W}" if "HttpOnly" in attrs else f"{R}✗{W}"
                samesite = attrs.get("SameSite", f"{R}Not Set{W}")
                print(
                    f"   {C}{name.ljust(20)}{W} Secure={secure} HttpOnly={httponly} SameSite={samesite}"
                )
                if output != "None":
                    result.setdefault("cookies", []).append(cookie)

    except Exception as exc:
        print(f"\n{R}[-] {C}Exception : {W}{exc}\n")
        if output != "None":
            result.update({"Exception": str(exc)})
        log_writer(f"[headers] Exception = {exc}")
    result.update({"exported": False})

    if output != "None":
        fname = f"{output['directory']}/headers.{output['format']}"
        output["file"] = fname
        data["module-headers"] = result
        export(output, data)
    log_writer("[headers] Completed")



#!/usr/bin/env python3

import sys

R = '\033[31m'  # red
G = '\033[32m'  # green
C = '\033[36m'  # cyan
W = '\033[0m'   # white
Y = '\033[33m'  # yellow


def export(output, data):
    if output['format'] != 'txt':
        print(f'{R}[-] {C}Invalid Output Format, Valid Formats : {W}txt')
        sys.exit()

    fname = output['file']
    with open(fname, 'w') as outfile:
        txt_export(data, outfile)


def txt_unpack(outfile, val):
    def write_item(item):
        if isinstance(item, list):
            outfile.write(f'{item[0]}\t{item[1]}\t\t{item[2]}\n')
        else:
            outfile.write(f'{item}\n')

    if isinstance(val, list):
        for item in val:
            write_item(item)

    elif isinstance(val, dict):
        for sub_key, sub_val in val.items():
            if sub_key == 'exported':
                continue
            if isinstance(sub_val, list):
                txt_unpack(outfile, sub_val)
            else:
                outfile.write(f'{sub_key}: {sub_val}\n')


def txt_export(data, outfile):
    for key, val in data.items():
        if key.startswith('module'):
            if not val['exported']:
                txt_unpack(outfile, val)
                val['exported'] = True
        elif key.startswith('Type'):
            outfile.write(f'\n{data[key]}\n')
            outfile.write(f'{"=" * len(data[key])}\n\n')
        else:
            outfile.write(f'{key}: {val}\n')



#!/usr/bin/env python3

import asyncio

from modules.write_log import log_writer

R = "\033[31m"
G = "\033[32m"
C = "\033[36m"
W = "\033[0m"
Y = "\033[33m"


async def leakix(hostname, api_key, session):
    subdomains = []
    error = None

    if api_key is not None:
        url = f"https://leakix.net/api/subdomains/{hostname}"
        header = {
            "api-key": api_key,
            "accept": "application/json",
        }

        try:
            async with session.get(url, headers=header) as resp:
                status = resp.status
                if status == 200:
                    json_data = await resp.json()
                    subdomains = [
                        entry["subdomain"]
                        for entry in json_data
                        if "subdomain" in entry
                    ]
                else:
                    error = f"Status : {status}"
                    log_writer(f"[leakix_subs] Status = {status}, expected 200")
        except asyncio.TimeoutError:
            error = "Request Timeout"
            log_writer(f"[leakix_subs] Exception = {error}")
        except Exception as exc:
            error = str(exc)
            log_writer(f"[leakix_subs] Exception = {exc}")
    else:
        error = "API key not configured"
        log_writer(f"[leakix_subs] {error}")

    log_writer("[leakix_subs] Completed")
    return "LeakIX", subdomains, error



#!/usr/bin/env python3

import asyncio
from json import loads

from modules.write_log import log_writer

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow


async def alienvault(hostname, api_key, session):
    error = None
    subdomains = []
    if api_key is not None:
        header = {"X-OTX-API-KEY": api_key}
        url = f"https://otx.alienvault.com/api/v1/indicators/domain/{hostname}/passive_dns"
        try:
            async with session.get(url, headers=header) as resp:
                status = resp.status
                if status == 200:
                    output = await resp.text()
                    json_data = loads(output)["passive_dns"]
                    for entry in json_data:
                        subdomains.append(entry["hostname"])
                else:
                    error = f"Status : {status}"
                    log_writer(f"[alienvault_subs] Status = {status}, expected 200")
                    log_writer(f"[alienvault_subs] Exception = {await resp.text()}")
        except asyncio.TimeoutError:
            error = "Request Timeout"
        except Exception as exc:
            error = exc
        log_writer(f"[alienvault_subs] Exception = {error}")
        log_writer("[alienvault_subs] Completed")
    else:
        error = "API key not configured"
        log_writer(f"[alienvault_subs] {error}")

    return "Alienvault", subdomains, error



#!/usr/bin/env python3
import asyncio
from json import loads

from modules.write_log import log_writer

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow


async def netlas(hostname, api_key, session):
    subdomains = []
    error = None

    if api_key is not None:
        url = "https://app.netlas.io/api/domains/download/"
        header = {"X-API-Key": api_key}
        payload = {
            "q": f"domain: *.{hostname}",
            "fields": ["domain"],
            "source_type": "include",
            "size": "200",
        }

        try:
            async with session.post(url, headers=header, data=payload) as resp:
                status = resp.status
                if status == 200:
                    json_data = loads(await resp.text())
                    for entry in json_data:
                        subdomain = entry["data"]["domain"]
                        subdomains.append(subdomain)
                else:
                    error = f"Status : {status}"
                    log_writer(f"[netlas_subs] Status = {status}, expected 200")
        except asyncio.TimeoutError:
            error = "Request Timeout"
            log_writer(f"[netlas_subs] Exception = {error}")
        except Exception as exc:
            error = exc
            log_writer(f"[netlas_subs] Exception = {exc}")
    else:
        error = "API key not configured"
        log_writer(f"[netlas_subs] {error}")
    log_writer("[netlas_subs] Completed")
    return "Netlas", subdomains, error



#!/usr/bin/env python3
import asyncio
from base64 import urlsafe_b64encode
from datetime import datetime, timedelta

from modules.write_log import log_writer

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow


async def hunter(hostname, api_key, session):
    subdomains = []
    error = None

    if api_key is not None:
        url = "https://api.hunter.how/search"
        query = f'domain.suffix=="{hostname}"'
        query64 = urlsafe_b64encode(query.encode("utf-8")).decode("ascii")
        start_year = datetime.today().year - 1
        start_month = datetime.today().month
        start_day = datetime.today().day
        try:
            start_date = datetime.strptime(
                f"{start_year}-{start_month}-{start_day}", "%Y-%m-%d"
            ).strftime("%Y-%m-%d")
        except ValueError:
            # handle leap year
            start_date = datetime.strptime(
                f"{start_year}-{start_month}-{start_day - 1}", "%Y-%m-%d"
            ).strftime("%Y-%m-%d")
        end_date = (datetime.today() - timedelta(days=2)).strftime("%Y-%m-%d")

        payload = {
            "api-key": api_key,
            "query": query64,
            "page": 1,
            "page_size": 1000,
            "start_time": start_date,
            "end_time": end_date,
        }
        try:
            async with session.get(url, params=payload) as resp:
                status = resp.status
                if status == 200:
                    json_data = await resp.json()
                    resp_code = json_data["code"]
                    if resp_code != 200:
                        resp_msg = json_data["message"]
                        error = f"{resp_code}, {resp_msg}"
                        log_writer(f"[hunter_subs] Status = {resp_code}, expected 200")
                    else:
                        subdomain_list = json_data["data"]["list"]
                        for entry in subdomain_list:
                            subdomains.append(entry["domain"])
                else:
                    error = f"Status : {status}"
                    log_writer(f"[hunter_subs] Status = {status}, expected 200")
        except asyncio.TimeoutError:
            error = "Request Timeout"
            log_writer(f"[hunter_subs] Exception = {error}")
        except Exception as exc:
            error = exc
            log_writer(f"[hunter_subs] Exception = {exc}")
    else:
        error = "API key not configured"
        log_writer(f"[hunter_subs] {error}")
    log_writer("[hunter_subs] Completed")

    return "Hunter", subdomains, error



#!/usr/bin/env python3

from json import loads
import modules.subdom as parent
from modules.write_log import log_writer

R = '\033[31m'  # red
G = '\033[32m'  # green
C = '\033[36m'  # cyan
W = '\033[0m'   # white
Y = '\033[33m'  # yellow


async def sonar(hostname, session):
	print(f'{Y}[!] {C}Requesting {G}Sonar{W}')
	url = f'https://sonar.omnisint.io/subdomains/{hostname}'
	try:
		async with session.get(url) as resp:
			status = resp.status
			if status == 200:
				json_data = await resp.text()
				json_read = loads(json_data)
				print(f'{G}[+] {Y}Sonar {W}found {C}{len(json_read)} {W}subdomains!')
				parent.found.extend(json_read)
			else:
				print(f'{R}[-] {C}Sonar Status : {W}{status}')
				log_writer(f'[sonar_subs] Status = {status}, expected 200')
	except Exception as exc:
		print(f'{R}[-] {C}Sonar Exception : {W}{exc}')
		log_writer(f'[sonar_subs] Exception = {exc}')
	log_writer('[sonar_subs] Completed')



#!/usr/bin/env python3

from json import loads
import modules.subdom as parent
from modules.write_log import log_writer

R = '\033[31m'  # red
G = '\033[32m'  # green
C = '\033[36m'  # cyan
W = '\033[0m'   # white
Y = '\033[33m'  # yellow


async def crtsh(hostname, session):
	print(f'{Y}[!] {C}Requesting {G}crt.sh{W}')
	url = f'https://crt.sh/?dNSName=%25.{hostname}&output=json'

	try:
		async with session.get(url) as resp:
			status = resp.status
			if status == 200:
				data = await resp.text()
				data_json = loads(data)
				tmp_list = []
				for entry in data_json:
					subdomain = entry['name_value']
					tmp_list.append(subdomain)
				print(f'{G}[+] {Y}crt.sh {W}found {C}{len(tmp_list)} {W}subdomains!')
				parent.found.extend(tmp_list)
			else:
				print(f'{R}[-] {C}HackerTarget Status : {W}{status}')
				log_writer(f'[htarget_subs] Status = {status}, expected 200')
	except Exception as exc:
		print(f'{R}[-] {C}crtsh Exception : {W}{exc}')
		log_writer(f'[crtsh_subs] Exception = {exc}')
	log_writer('[crtsh_subs] Completed')



#!/usr/bin/env python3

import asyncio
from json import dump, loads
from os import environ
from re import match

import aiohttp

from modules.export import export
from modules.subdomain_modules.alienvault_subs import alienvault
from modules.subdomain_modules.anubis_subs import anubisdb
from modules.subdomain_modules.bevigil_subs import bevigil
from modules.subdomain_modules.certspot_subs import certspot
from modules.subdomain_modules.chaos_subs import chaos
from modules.subdomain_modules.github_subs import github
from modules.subdomain_modules.htarget_subs import hackertgt
from modules.subdomain_modules.hunter_subs import hunter
from modules.subdomain_modules.leakix_subs import leakix
from modules.subdomain_modules.netlas_subs import netlas
from modules.subdomain_modules.shodan_subs import shodan
from modules.subdomain_modules.urlscan_subs import urlscan
from modules.subdomain_modules.virustotal_subs import virust
from modules.subdomain_modules.wayback_subs import machine
from modules.subdomain_modules.zoomeye_subs import zoomeye
from modules.write_log import log_writer

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow
HEADER = "\033[1;35m"  # bold magenta

found = []


def migrate_config_file(config_path):
    current_schema = {
        "shodan": None,
        "zoomeye": None,
        "virustotal": None,
        "alienvault": None,
        "hunter": None,
        "chaos": None,
        "leakix": None,
        "netlas": None,
        "bevigil": None,
        "github": None,
    }

    try:
        with open(config_path, "r") as f:
            user_data = loads(f.read())
    except FileNotFoundError:
        user_data = {}

    missing_keys = {k: v for k, v in current_schema.items() if k not in user_data}

    if missing_keys:
        user_data.update(missing_keys)

        with open(config_path, "w") as f:
            dump(user_data, f, indent=4)
        print(
            f"[*] Migrated config: Added {len(missing_keys)} new API key slots to {config_path}"
        )

    return user_data


async def query(hostname, tout, api_keys):
    timeout = aiohttp.ClientTimeout(total=tout)
    async with aiohttp.ClientSession(timeout=timeout) as session:
        results = await asyncio.gather(
            alienvault(hostname, api_keys["alienvault"], session),
            anubisdb(hostname, session),
            bevigil(hostname, api_keys["bevigil"], session),
            certspot(hostname, session),
            hackertgt(hostname, session),
            hunter(hostname, api_keys["hunter"], session),
            netlas(hostname, api_keys["netlas"], session),
            shodan(hostname, api_keys["shodan"], session),
            urlscan(hostname, session),
            virust(hostname, api_keys["virustotal"], session),
            zoomeye(hostname, api_keys["zoomeye"], session),
            chaos(hostname, api_keys["chaos"], session),
            leakix(hostname, api_keys["leakix"], session),
            github(hostname, api_keys["github"], session),
            machine(hostname, session),
        )
    await session.close()
    return results


def subdomains(hostname, tout, output, data, conf_path):
    global found
    result = {}
    api_keys = {}

    print(f"\n{HEADER}━━━ SubDomain Enum {'━' * 30}{W}\n")

    migrate_config_file(f"{conf_path}/keys.json")

    with open(f"{conf_path}/keys.json", "r") as keyfile:
        json_read = keyfile.read()

    keys_json = loads(json_read)
    api_keys["alienvault"] = environ.get("FR_ALIENVAULT_KEY") or keys_json.get(
        "alienvault"
    )
    api_keys["bevigil"] = environ.get("FR_BEVIGIL_KEY") or keys_json.get("bevigil")
    api_keys["hunter"] = environ.get("FR_HUNTER_KEY") or keys_json.get("hunter")
    api_keys["netlas"] = environ.get("FR_NETLAS_KEY") or keys_json.get("netlas")
    api_keys["shodan"] = environ.get("FR_SHODAN_KEY") or keys_json.get("shodan")
    api_keys["virustotal"] = environ.get("FR_VT_KEY") or keys_json.get("virustotal")
    api_keys["zoomeye"] = environ.get("FR_ZOOMEYE_KEY") or keys_json.get("zoomeye")
    api_keys["chaos"] = environ.get("FR_CHAOS_KEY") or keys_json.get("chaos")
    api_keys["leakix"] = environ.get("FR_LEAKIX_KEY") or keys_json.get("leakix")
    api_keys["github"] = environ.get("FR_GITHUB_KEY") or keys_json.get("github")

    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    results = loop.run_until_complete(query(hostname, tout, api_keys))
    for name, subs, error in results:
        if error is None:
            found.extend(subs)
        if subs:
            print(f"{G}[+] {C}{name.ljust(15)}{W} : {len(subs)}")
        elif error is not None:
            if error.startswith("API key not configured"):
                print(f"{Y}[!] {C}{name.ljust(15)}{W} : {error}")
            else:
                print(f"{R}[-] {C}{name.ljust(15)}{W} : {error}")
    loop.close()

    found = [item for item in found if item.endswith(hostname)]
    valid = r"^[A-Za-z0-9._~()'!*:@,;+?-]*$"
    found = [item for item in found if match(valid, item)]
    found = set(found)
    total = len(found)

    if found:
        print(f"\n{G}[+]{W} {'Total Unique'.ljust(15)} : {total}\n")
        for url in enumerate(list(found)[:20]):
            print(url[1])

        if len(found) > 20:
            print(f"\n{C}[*]{W} Results truncated...")

    if output != "None":
        result["Links"] = list(found)
        result.update({"exported": False})
        data["module-Subdomain Enumeration"] = result
        fname = f"{output['directory']}/subdomains.{output['format']}"
        output["file"] = fname
        export(output, data)
    log_writer("[subdom] Completed")



#!/usr/bin/env python3

from json import loads
import modules.subdom as parent
from modules.write_log import log_writer

R = '\033[31m'  # red
G = '\033[32m'  # green
C = '\033[36m'  # cyan
W = '\033[0m'   # white
Y = '\033[33m'  # yellow


async def thminer(hostname, session):
	print(f'{Y}[!] {C}Requesting {G}ThreatMiner{W}')
	url = 'https://api.threatminer.org/v2/domain.php'
	thm_params = {
		'q': hostname,
		'rt': '5'
	}
	try:
		async with session.get(url, params=thm_params) as resp:
			status = resp.status
			if status == 200:
				output = await resp.text()
				json_out = loads(output)
				subd = json_out['results']
				print(f'{G}[+] {Y}ThreatMiner {W}found {C}{len(subd)} {W}subdomains!')
				parent.found.extend(subd)
			else:
				print(f'{R}[-] {C}ThreatMiner Status : {W}{status}')
				log_writer(f'[thminer_subs] Status = {status}, expected 200')
	except Exception as exc:
		print(f'{R}[-] {C}ThreatMiner Exception : {W}{exc}')
		log_writer(f'[thminer_subs] Exception = {exc}')
	log_writer('[thminer_subs] Completed')



#!/usr/bin/env python3

import asyncio

from modules.export import export
from modules.write_log import log_writer

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow
HEADER = "\033[1;35m"  # bold magenta

counter = 0
port_list = {
    1: "tcpmux",
    9: "Discard",
    15: "netstat",
    20: "FTP-CLI",
    21: "FTP",
    22: "SSH",
    23: "Telnet",
    25: "SMTP",
    26: "rsftp",
    53: "DNS",
    67: "DHCP (Server)",
    68: "DHCP (Client)",
    69: "TFTP",
    80: "HTTP",
    110: "POP3",
    119: "NNTP",
    123: "NTP",
    135: "Microsoft RPC",
    137: "NetBIOS Name Service",
    138: "NetBIOS Datagram Service",
    139: "NetBIOS Session Service",
    143: "IMAP",
    161: "SNMP",
    162: "SNMP Trap",
    179: "BGP",
    194: "IRC",
    389: "LDAP",
    443: "HTTPS",
    445: "Microsoft-DS",
    465: "SMTPS",
    515: "LPD",
    520: "RIP",
    554: "RTSP (Real-Time Streaming)",
    587: "SMTP (Submission)",
    631: "IPP (CUPS)",
    636: "LDAPS",
    873: "rsync",
    990: "FTPS",
    993: "IMAPS",
    995: "POP3S",
    1024: "Dynamic/Private",
    1080: "Socks Proxy",
    1194: "OpenVPN",
    1433: "Microsoft SQL Server",
    1434: "Microsoft SQL Monitor",
    1521: "Oracle DB",
    1701: "L2TP",
    1723: "PPTP",
    1883: "MQTT",
    2000: "Cisco-sccp",
    2049: "NFS",
    2222: "EtherNetIP-1",
    2375: "Docker REST API",
    2376: "Docker REST API (TLS)",
    2483: "Oracle DB",
    2484: "Oracle DB (TLS)",
    3000: "Grafana",
    3306: "MySQL",
    3389: "RDP",
    3690: "Subversion",
    4373: "Remote Authenticated Command",
    4443: "HTTPS-Alt",
    4444: "Metasploit",
    4567: "MySQL Group Replication",
    4786: "Cisco Smart Install",
    5060: "SIP",
    5432: "PostgreSQL",
    5672: "RabbitMQ",
    5900: "VNC",
    5938: "TeamViewer",
    5984: "CouchDB",
    6379: "Redis",
    6443: "Kubernetes API",
    6667: "IRC",
    7000: "Couchbase",
    7200: "Hazelcast",
    8000: "HTTP-Alt",
    8008: "HTTP-Alt",
    8080: "HTTP-Proxy",
    8081: "SonarQube",
    8086: "InfluxDB",
    8088: "Kibana",
    8181: "HTTP-Alt",
    8443: "HTTPS-Alt",
    8444: "Jenkins",
    8888: "HTTP-Alt",
    9000: "SonarQube",
    9090: "Openfire",
    9092: "Kafka",
    9093: "Prometheus Alertmanager",
    9200: "Elasticsearch",
    9300: "Elasticsearch",
    9418: "Git",
    9990: "JBoss Management",
    9993: "Unreal Tournament",
    9999: "NMAP",
    10000: "Webmin",
    10050: "Zabbix Agent",
    10051: "Zabbix Server",
    11211: "Memcached",
    11300: "Beanstalkd",
    25565: "Minecraft",
    27015: "Source Engine Games",
    27017: "MongoDB",
    27018: "MongoDB",
    5044: "Logstash",
    50000: "SAP",
    50030: "Hadoop",
    50070: "Hadoop",
    5555: "Open Remote",
    61616: "ActiveMQ",
}


async def insert(queue):
    for port in port_list:
        await queue.put(port)


async def consumer(queue, ip_addr, result):
    global counter
    while True:
        port = await queue.get()
        await sock_conn(ip_addr, port, result)
        queue.task_done()
        counter += 1
        print(
            f"{Y}[!]{W} Scanning : {counter}/{len(port_list)}",
            end="\r",
            flush=True,
        )


async def run(ip_addr, result, threads):
    queue = asyncio.Queue(maxsize=threads)
    distrib = asyncio.create_task(insert(queue))
    workers = [
        asyncio.create_task(consumer(queue, ip_addr, result)) for _ in range(threads)
    ]

    await asyncio.gather(distrib)
    await queue.join()
    for worker in workers:
        worker.cancel()


def scan(ip_addr, output, data, threads):
    result = {}
    result["ports"] = []
    print(f"\n{HEADER}━━━ Port Scan {'━' * 30}{W}\n")
    print(f"{C}[*]{W} Scanning Top {G}100+{W} Ports With {G}{threads}{W} Threads...\n")

    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    loop.run_until_complete(run(ip_addr, result, threads))
    loop.close()

    print()

    if output != "None":
        ps_output(output, data, result)
    log_writer("[portscan] Completed")


async def sock_conn(ip_addr, port, result):
    try:
        connector = asyncio.open_connection(ip_addr, port)
        await asyncio.wait_for(connector, 1)
        port_name = port_list[port]
        print(f"\r\033[K{G}[+]{W} {str(port).ljust(6)} {port_name}")
        result["ports"].append(f"{port} ({port_name})")
        return True
    except TimeoutError:
        return False
    except Exception:
        pass


def ps_output(output, data, result):
    data["module-Port Scan"] = result
    result.update({"exported": False})
    fname = f"{output['directory']}/ports.{output['format']}"
    output["file"] = fname
    export(output, data)



#!/usr/bin/env python3

import asyncio
import socket
from datetime import date
from random import choices
from string import ascii_letters, digits
from urllib.parse import urlparse

import aiohttp

import settings as config
from modules.export import export
from modules.write_log import log_writer

R = "\033[31m"  # red
G = "\033[32m"  # green
C = "\033[36m"  # cyan
W = "\033[0m"  # white
Y = "\033[33m"  # yellow
HEADER = "\033[1;35m"  # bold magenta

header = {"User-Agent": f"FinalRecon/{config.version}"}
count = 0
wm_count = 0
exc_count = 0
found = []
curr_yr = date.today().year
last_yr = curr_yr - 1
stop_event = asyncio.Event()
length_counts = {}


async def fetch(url, session, redir):
    global exc_count
    if stop_event.is_set():
        return
    try:
        async with session.get(url, headers=header, allow_redirects=redir) as response:
            cont_len = len(await response.read())
            cont_type = response.headers.get("Content-Type", "").split(";")[0].strip()
            location = response.headers.get("Location", "")
            return response.status, cont_type, cont_len, location
    except asyncio.TimeoutError:
        exc_count += 1
        log_writer(f"[dirrec.fetch] Exception : Request Timed Out [{url}]")
        return None, None, None, None
    except aiohttp.ClientConnectionError as cce_exc:
        exc_count += 1
        log_writer(f"[dirrec.fetch] Exception : {cce_exc} [{url}]")
        return None, None, None, None
    except aiohttp.ClientError as exc:
        exc_count += 1
        log_writer(f"[dirrec.fetch] Exception : {exc} [{url}]")
        return None, None, None, None
    except Exception as err:
        exc_count += 1
        log_writer(f"[dirrec.fetch] Exception : {err} [{url}]")
        return None, None, None, None


async def insert(queue, filext, target, wdlist, redir):
    if not filext:
        url = target + "/{}"
        with open(wdlist, "r", errors="replace") as wordlist:
            for word in wordlist:
                word = word.strip()
                await queue.put([url.format(word), redir, len(word)])
                await asyncio.sleep(0)
    else:
        if "," in filext:
            filext = filext.split(",")
        else:
            filext = [filext]
        with open(wdlist, "r", errors="replace") as wordlist:
            for word in wordlist:
                for ext in filext:
                    ext = ext.strip()
                    url = target + "/{}." + ext
                    word = word.strip()
                    await queue.put([url.format(word), redir, len(word)])
                    await asyncio.sleep(0)


async def consumer(
    queue,
    target,
    session,
    redir,
    total_num_words,
    use_cont_len,
    dir_in_content,
    exist_len,
    exist_location,
):
    global count
    while True:
        values = await queue.get()
        if stop_event.is_set():
            queue.task_done()
            continue
        url = values[0]
        redir = values[1]
        dir_len = values[2]
        status, content_type, content_len, location = await fetch(url, session, redir)
        if status is not None:
            await filter_out(
                target,
                url,
                status,
                content_type,
                content_len,
                location,
                use_cont_len,
                dir_in_content,
                dir_len,
                exist_len,
                exist_location,
            )
        queue.task_done()
        count += 1

        print(
            f"\r\033[K{C}[*]{W} Requests : {count}/{total_num_words}",
            end="\r",
            flush=True,
        )


async def run(target, threads, tout, wdlist, redir, sslv, filext, total_num_words):
    use_cont_len = False
    dir_in_content = False
    len_test_res = []
    queue = asyncio.Queue(maxsize=threads)

    conn = aiohttp.TCPConnector(limit=threads, family=socket.AF_INET, verify_ssl=sslv)
    timeout = aiohttp.ClientTimeout(total=tout, sock_connect=tout, sock_read=tout)
    async with aiohttp.ClientSession(connector=conn, timeout=timeout) as session:
        print(f"{C}[*]{W} Probing for phantom URLs...")
        rand_urls = [
            "".join(choices(ascii_letters + digits, k=k)) for k in (10, 6, 8, 7, 9)
        ]
        for item in rand_urls:
            rand_url = f"{target}/{item}"
            exist_sc, con_type, exist_len, exist_location = await fetch(
                rand_url, session, True
            )
            len_test_res.append((item, exist_sc, exist_len))

        successful = [r for r in len_test_res if r[1] == 200]
        if successful:
            print(f"{Y}[!]{W} Response 200 OK for non existent URLs...")
            use_cont_len = True

            if len(successful) >= 2:
                cont_len1, dir_len1 = successful[0][2], len(successful[0][0])
                cont_len2, dir_len2 = successful[1][2], len(successful[1][0])
                if cont_len1 != cont_len2:
                    if (cont_len1 - dir_len1) == (cont_len2 - dir_len2):
                        print(f"{C}[*]{W} Content contains directory name...")
                        dir_in_content = True

            lengths = sorted(r[2] for r in successful)
            exist_len = lengths[len(lengths) // 2]
            if dir_in_content:
                exist_len -= len(successful[lengths.index(exist_len)][0])

            print(f"{C}[*]{W} Filtering content length for accuracy...")

        print()
        print(f"{Y}{'STATUS'} {'LENGTH'.ljust(10)} {'TYPE'.ljust(30)} URL{W}")

        distrib = asyncio.create_task(insert(queue, filext, target, wdlist, redir))
        workers = [
            asyncio.create_task(
                consumer(
                    queue,
                    target,
                    session,
                    redir,
                    total_num_words,
                    use_cont_len,
                    dir_in_content,
                    exist_len,
                    exist_location,
                )
            )
            for _ in range(threads)
        ]

        await asyncio.gather(distrib)
        await queue.join()
        print()

        for worker in workers:
            worker.cancel()


def normalize_location(location):
    try:
        return urlparse(location).path
    except Exception as exc:
        print(exc)


async def filter_out(
    target,
    url,
    status,
    cont_type,
    length,
    location,
    use_cont_len,
    dir_in_content,
    dir_len,
    exist_len,
    exist_location,
):
    global found

    def log_and_append(status, length, url, color):
        found.append((url, status, length))
        print(
            f"{color}{str(status).ljust(6)}{W} {str(length).ljust(10)} {cont_type.ljust(30)} {url}",
        )

    if status in {301, 302, 303, 307, 308}:
        location_key = normalize_location(location)
        length_counts[location_key] = length_counts.get(location_key, 0) + 1
        if length_counts[location_key] == 10:
            print(
                f"\r\033[K{Y}[!]{W} Auto-filtering redirect to {location_key} — possible soft-404"
            )
        if length_counts[location_key] >= 10:
            return
    else:
        if length > 0:
            length_key = round(length, -1)
            length_counts[length_key] = length_counts.get(length_key, 0) + 1
            if length_counts[length_key] == 10:
                print(
                    f"\r\033[K{Y}[!]{W} Auto-filtering length ~{length_key} — possible soft-404"
                )
            if length_counts[length_key] >= 10:
                found[:] = [
                    f
                    for f in found
                    if not (round(f[2], -1) == length_key and f[1] == 200)
                ]
                return

    if status in {200}:
        compare_len = length
        if use_cont_len:
            if dir_in_content:
                compare_len = length - dir_len
            if round(compare_len, -1) != round(exist_len, -1):
                log_and_append(status, length, url, G)
        else:
            if str(url) != target + "/":
                log_and_append(status, length, url, G)
    elif status in {301, 302, 303, 307, 308}:
        if exist_location and normalize_location(location) == normalize_location(
            exist_location
        ):
            return
        log_and_append(status, length, url, Y)
    elif status in {403}:
        log_and_append(status, length, url, R)
    elif status in {429}:
        print(f"{R}[-]{W} Status 429, getting ratelimited, stopping...")
        stop_event.set()


def dir_output(output, data):
    result = {}
    status_map = {
        200: "Status 200",
        403: "Status 403",
        301: "Status 301",
        302: "Status 302",
        303: "Status 303",
        307: "Status 307",
        308: "Status 308",
    }
    for entry in found:
        if entry is None or output == "None":
            continue
        status = entry[1]
        status_str = status_map.get(status)
        if status_str:
            result.setdefault(status_str, []).append(f"{status}, {entry[0]}")

    print(f"\n{G}[+]{W} Directories Found   : {len(found)}\n")
    print(f"{C}[*]{W} Exceptions          : {exc_count}")

    if output != "None":
        result.update({"exported": False})
        data["module-Directory Search"] = result
        fname = f"{output['directory']}/directory_enum.{output['format']}"
        output["file"] = fname
        export(output, data)


def hammer(target, threads, tout, wdlist, redir, sslv, output, data, filext):
    print(f"\n{HEADER}━━━ Dir Enum {'━' * 30}{W}\n")
    print(f"{G}Threads          : {W}{threads}")
    print(f"{G}Timeout          : {W}{tout}")
    print(f"{G}Wordlist         : {W}{wdlist}")
    print(f"{G}Allow Redirects  : {W}{redir}")
    print(f"{G}SSL Verification : {W}{sslv}")
    with open(wdlist, "r", errors="replace") as wordlist:
        num_words = sum(1 for i in wordlist)
    print(f"{G}Wordlist Size    : {W}{num_words}")
    print(f"{G}File Extensions  : {W}{filext}\n")
    if len(filext) != 0:
        total_num_words = num_words * (len(filext.split(",")) + 1)
    else:
        total_num_words = num_words

    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    loop.run_until_complete(
        run(target, threads, tout, wdlist, redir, sslv, filext, total_num_words)
    )
    dir_output(output, data)
    loop.close()
    log_writer("[dirrec] Completed")
